@@ -71,7 +71,7 @@ def init(config_object: Optional[Any] = None,
 
     This function must be called at the very beginning of code,
     otherwise the get_bot() function will return None and nothing
-    is gonna work properly.
+    will work properly.
 
     :param config_object: configuration object
     """
@@ -140,9 +140,10 @@ from .plugin import (on_command, on_natural_language, on_notice, on_request,
                      load_plugin, load_plugins, load_builtin_plugins,
                      get_loaded_plugins)
 from .message import message_preprocessor, Message, MessageSegment
-from .natural_language import NLPSession, NLPResult, IntentCommand
+from .natural_language import NLPSession, IntentCommand
 from .notice_request import NoticeSession, RequestSession
 from .helpers import context_id
+from .permission import SenderRoles
 
 __all__ = [
     'NoneBot',
@@ -165,11 +166,11 @@ __all__ = [
     'CommandGroup',
     'on_natural_language',
     'NLPSession',
-    'NLPResult',
     'IntentCommand',
     'on_notice',
     'NoticeSession',
     'on_request',
     'RequestSession',
     'context_id',
+    'SenderRoles',
 ]
