@@ -1,7 +1,7 @@
 import re
 import asyncio
 import warnings
-from datetime import datetime
+from datetime import datetime, timedelta
 from functools import partial
 from typing import (NoReturn, Tuple, Union, Iterable, Any, Optional, List, Dict,
                     Awaitable, Pattern, Type)
@@ -9,32 +9,98 @@ from typing import (NoReturn, Tuple, Union, Iterable, Any, Optional, List, Dict,
 from aiocqhttp import Event as CQEvent
 from aiocqhttp.message import Message
 
-from nonebot import NoneBot, permission as perm
+from nonebot import NoneBot
 from nonebot.command.argfilter import ValidateError
 from nonebot.helpers import context_id, send, render_expression
 from nonebot.log import logger
 from nonebot.session import BaseSession
 from nonebot.typing import (CommandName_T, CommandArgs_T, CommandHandler_T,
-                            Message_T, State_T, Filter_T, Patterns_T)
+                            Message_T, PermissionPolicy_T, State_T, Filter_T, Patterns_T)
+from nonebot.permission import check_permission
 
 # key: context id
 # value: CommandSession object
 _sessions = {}  # type: Dict[str, "CommandSession"]
 
 
+class CommandInterrupt(Exception):
+    """INTERNAL API"""
+    pass
+
+
+class _YieldException(CommandInterrupt):
+    """
+    Raised by command.run(), session indicating that the waiting session
+    will resume and current execution path should return immediately.
+    """
+    pass
+
+
+class _PauseException(CommandInterrupt):
+    """
+    Raised by session.pause() indicating that the command session
+    should be paused to ask the user for some arguments.
+    """
+    pass
+
+
+class _FinishException(CommandInterrupt):
+    """
+    Raised by session.finish() indicating that the command session
+    should be stopped and removed.
+    """
+
+    def __init__(self, result: bool = True):
+        """
+        :param result: succeeded to call the command
+        """
+        self.result = result
+
+
+class SwitchException(CommandInterrupt):
+    """
+    INTERNAL API
+
+    Raised by session.switch() indicating that the command session
+    should be stopped and replaced with a new one (going through
+    handle_message() again).
+
+    Since the new message will go through handle_message() again,
+    the later function should be notified. So this exception is
+    intended to be propagated to handle_message().
+    """
+
+    def __init__(self, new_message: Message):
+        """
+        :param new_message: new message which should be placed in event
+        """
+        self.new_message = new_message
+
+
 class Command:
-    __slots__ = ('name', 'func', 'permission', 'only_to_me', 'privileged',
-                 'args_parser_func', 'session_class')
+    """
+    INTERNAL API
+
+    Note ... (Ellipsis) is a valid value for field expire_timeout and run_timeout.
+    However I cannot type it until Python 3.10. see bugs.python.org/issue41810.
+    """
+    __slots__ = ('name', 'func', 'only_to_me', 'privileged', 'args_parser_func',
+                 'permission', 'expire_timeout', 'run_timeout', 'session_class')
 
     def __init__(self, *, name: CommandName_T, func: CommandHandler_T,
-                 permission: int, only_to_me: bool, privileged: bool,
+                 only_to_me: bool, privileged: bool,
+                 permission: PermissionPolicy_T,
+                 expire_timeout: Optional[timedelta],
+                 run_timeout: Optional[timedelta],
                  session_class: Optional[Type['CommandSession']]):
         self.name = name
         self.func = func
-        self.permission = permission
         self.only_to_me = only_to_me
         self.privileged = privileged
         self.args_parser_func: Optional[CommandHandler_T] = None
+        self.permission = permission  # includes EllipsisType
+        self.expire_timeout = expire_timeout  # includes EllipsisType
+        self.run_timeout = run_timeout  # includes EllipsisType
         self.session_class = session_class
 
     async def run(self,
@@ -101,12 +167,15 @@ class Command:
                         session.current_key not in session.state:
                     # args_parser_func didn't set state, here we set it
                     session.state[session.current_key] = session.current_arg
-
-            await self.func(session)
+            if session.waiting:
+                session._future.set_result(True)
+                raise _YieldException()
+            else:
+                await self.func(session)
             return True
         return False
 
-    async def _check_perm(self, session) -> bool:
+    async def _check_perm(self, session: 'CommandSession') -> bool:
         """
         Check if the session has sufficient permission to
         call the command.
@@ -114,8 +183,9 @@ class Command:
         :param session: CommandSession object
         :return: the session has the permission
         """
-        return await perm.check_permission(session.bot, session.event,
-                                           self.permission)
+        return await check_permission(session.bot, session.event,
+            self.permission if self.permission is not ...
+                else session.bot.config.DEFAULT_COMMAND_PERMISSION)
 
     def args_parser(self, parser_func: CommandHandler_T) -> CommandHandler_T:
         """
@@ -137,7 +207,7 @@ class CommandManager:
     _commands = {}  # type: Dict[CommandName_T, Command]
     _aliases = {}  # type: Dict[str, Command]
     _switches = {}  # type: Dict[Command, bool]
-    _patterns = {}  # type: Dict[Pattern, Command]
+    _patterns = {}  # type: Dict[Pattern[str], Command]
 
     def __init__(self):
         self.commands = CommandManager._commands.copy()
@@ -226,7 +296,7 @@ class CommandManager:
         
         Args:
             aliases (Union[Iterable[str], str]): Command aliases
-            cmd_name (Command): Command
+            cmd (Command): Command
         """
         if isinstance(aliases, str):
             aliases = (aliases,)
@@ -261,64 +331,12 @@ class CommandManager:
                 continue
             cls._patterns[pattern] = cmd
 
-    def _add_command_to_tree(self, cmd_name: CommandName_T, cmd: Command,
-                             tree: Dict[str, Union[Dict, Command]]) -> None:
-        """Add command to the target command tree.
-        
-        Args:
-            cmd_name (CommandName_T): Name of the command
-            cmd (Command): Command object
-            tree (Dict[str, Union[Dict, Command]): Target command tree
-        """
-        current_parent = tree
-        for parent_key in cmd_name[:-1]:
-            current_parent[parent_key] = current_parent.get(parent_key) or {}
-            current_parent = current_parent[parent_key]
-
-            if not isinstance(current_parent, dict):
-                warnings.warn(f"{current_parent} is not a registry dict")
-                return
-        if cmd_name[-1] in current_parent:
-            warnings.warn(f"There is already a command named {cmd_name}")
-            return
-        current_parent[cmd_name[-1]] = cmd
-
-    def _generate_command_tree(
-        self, commands: Dict[CommandName_T,
-                             Command]) -> Dict[str, Union[Dict, Command]]:
-        """Generate command tree from commands dictionary.
-        
-        Args:
-            commands (Dict[CommandName_T, Command]): Dictionary of commands
-        
-        Returns:
-            Dict[str, Union[Dict, "Command"]]: Command tree
-        """
-        cmd_tree = {}  # type: Dict[str, Union[Dict, "Command"]]
-        for cmd_name, cmd in commands.items():
-            self._add_command_to_tree(cmd_name, cmd, cmd_tree)
-        return cmd_tree
-
     def _find_command(self, name: Union[str,
                                         CommandName_T]) -> Optional[Command]:
         cmd_name = (name,) if isinstance(name, str) else name
         if not cmd_name:
             return None
 
-        # cmd_tree = self._generate_command_tree({
-        #     name: cmd
-        #     for name, cmd in self.commands.items()
-        #     if self.switches.get(cmd, True)
-        # })
-        # for part in cmd_name[:-1]:
-        #     if part not in cmd_tree or not isinstance(
-        #             cmd_tree[part],  #type: ignore
-        #             dict):
-        #         return None
-        #     cmd_tree = cmd_tree[part]  # type: ignore
-
-        # cmd = cmd_tree.get(cmd_name[-1])  # type: ignore
-        # return cmd if isinstance(cmd, Command) else None
         cmd = {
             name: cmd
             for name, cmd in self.commands.items()
@@ -427,54 +445,11 @@ class CommandManager:
             state)
 
 
-class CommandInterrupt(Exception):
-    pass
-
-
-class _PauseException(CommandInterrupt):
-    """
-    Raised by session.pause() indicating that the command session
-    should be paused to ask the user for some arguments.
-    """
-    pass
-
-
-class _FinishException(CommandInterrupt):
-    """
-    Raised by session.finish() indicating that the command session
-    should be stopped and removed.
-    """
-
-    def __init__(self, result: bool = True):
-        """
-        :param result: succeeded to call the command
-        """
-        self.result = result
-
-
-class SwitchException(CommandInterrupt):
-    """
-    Raised by session.switch() indicating that the command session
-    should be stopped and replaced with a new one (going through
-    handle_message() again).
-
-    Since the new message will go through handle_message() again,
-    the later function should be notified. So this exception is
-    intended to be propagated to handle_message().
-    """
-
-    def __init__(self, new_message: Message):
-        """
-        :param new_message: new message which should be placed in event
-        """
-        self.new_message = new_message
-
-
 class CommandSession(BaseSession):
     __slots__ = ('cmd', 'current_key', 'current_arg_filters',
                  '_current_send_kwargs', 'current_arg', '_current_arg_text',
                  '_current_arg_images', '_state', '_last_interaction',
-                 '_running', '_run_future')
+                 '_running', '_run_future', '_future')
 
     def __init__(self,
                  bot: NoneBot,
@@ -509,6 +484,7 @@ class CommandSession(BaseSession):
 
         self._last_interaction = None  # last interaction time of this session
         self._running = False
+        self._future: Optional[asyncio.Future] = None
 
     @property
     def state(self) -> State_T:
@@ -520,29 +496,48 @@ class CommandSession(BaseSession):
         """
         return self._state
 
-    @property
-    def args(self) -> CommandArgs_T:
-        """Deprecated. Use `session.state` instead."""
-        return self.state
-
     @property
     def running(self) -> bool:
+        """INTERNAL API"""
         return self._running
 
     @running.setter
     def running(self, value) -> None:
+        """INTERNAL API"""
         if self._running is True and value is False:
             # change status from running to not running, record the time
             self._last_interaction = datetime.now()
         self._running = value
 
+    @property
+    def waiting(self) -> bool:
+        """INTERNAL API"""
+        return self._future is not None and not self._future.done()
+
+    @property
+    def expire_timeout(self) -> Optional[timedelta]:
+        """INTERNAL API"""
+        if self.cmd.expire_timeout is not ...:
+            return self.cmd.expire_timeout
+        return self.bot.config.SESSION_EXPIRE_TIMEOUT
+
+    @property
+    def run_timeout(self) -> Optional[timedelta]:
+        """INTERNAL API"""
+        if self.cmd.run_timeout is not ...:
+            return self.cmd.run_timeout
+        return self.bot.config.SESSION_RUN_TIMEOUT
+
     @property
     def is_valid(self) -> bool:
-        """Check if the session is expired or not."""
-        if self.bot.config.SESSION_EXPIRE_TIMEOUT and \
-                self._last_interaction and \
-                datetime.now() - self._last_interaction > \
-                self.bot.config.SESSION_EXPIRE_TIMEOUT:
+        """
+        INTERNAL API
+
+        Check whether the session has expired or not.
+        """
+        tm = self.expire_timeout
+        if tm and self._last_interaction and \
+            datetime.now() - self._last_interaction > tm:
             return False
         return True
 
@@ -586,6 +581,8 @@ class CommandSession(BaseSession):
                 *,
                 current_arg: Optional[str] = '') -> None:
         """
+        INTERNAL API
+
         Refill the session with a new message event.
 
         :param event: new message event
@@ -613,6 +610,7 @@ class CommandSession(BaseSession):
         :param key: argument key
         :param prompt: prompt to ask the user
         :param arg_filters: argument filters for the next user input
+        :param kwargs: other keyword arguments used in BeseSession.send()
         :return: the argument value
         """
         if key in self.state:
@@ -623,27 +621,90 @@ class CommandSession(BaseSession):
         self._current_send_kwargs = kwargs
         self.pause(prompt, **kwargs)
 
-    def get_optional(self,
-                     key: str,
-                     default: Optional[Any] = None) -> Optional[Any]:
+    async def aget(self,
+                   key: str = ...,
+                   *,
+                   prompt: Optional[Message_T] = None,
+                   arg_filters: Optional[List[Filter_T]] = None,
+                   force_update: bool = ...,
+                   **kwargs) -> Any:
         """
-        Simply get a argument with given key.
+        Get an argument with a given key.
 
-        Deprecated. Use `session.state.get()` instead.
+        If the argument does not exist in the current session,
+        the current coroutine yields, and the caller of
+        the command will know it should keep the session for
+        further interaction with the user.
+
+        :param key: argument key
+        :param prompt: prompt to ask the user
+        :param arg_filters: argument filters for the next user input
+        :param force_update: true to ignore the current argument
+        :param kwargs: other keyword arguments used in BeseSession.send()
+        :return: the argument value
         """
-        return self.state.get(key, default)
+        if key is ...:
+            key = '__default_argument'
+            force_update = True
+        if key in self.state:
+            if force_update is True:
+                del self.state[key]
+            else:
+                return self.state[key]
+
+        self.current_key = key
+        self.current_arg_filters = arg_filters
+        self._current_send_kwargs = kwargs
+        await self.apause(prompt, **kwargs)
+
+        return self.state.get(key, self.current_arg)
 
     def pause(self, message: Optional[Message_T] = None, **kwargs) -> NoReturn:
-        """Pause the session for further interaction."""
+        """
+        Pause the session for further interaction. This function never returns.
+
+        :param message: message to send to the user
+        :param kwargs: other keyword arguments used in BeseSession.send()
+        """
         if message:
             self._run_future(self.send(message, **kwargs))
-        raise _PauseException
+        self._raise(_PauseException())
+
+    async def apause(self, message: Optional[Message_T] = None, **kwargs) -> None:
+        """
+        Pause the session for further interaction. The control flow will pick
+        up where it is left over when this command session is recalled.
+
+        :param message: message to send to the user
+        :param kwargs: other keyword arguments used in BeseSession.send()
+        """
+        if message:
+            self._run_future(self.send(message, **kwargs))
+        if self.waiting:
+            return
+        while True:
+            try:
+                self._future = asyncio.get_event_loop().create_future()
+                self.running = False
+                timeout_opt = self.expire_timeout
+                timeout = timeout_opt.total_seconds() if timeout_opt else None
+                await asyncio.wait_for(self._future, timeout)
+                break
+            except _PauseException:
+                continue
+            except (_FinishException, asyncio.TimeoutError):
+                raise
 
     def finish(self, message: Optional[Message_T] = None, **kwargs) -> NoReturn:
-        """Finish the session."""
+        """
+        Finish the session. This function never returns.
+
+        :param message: message to send to the user when this command exits
+        :param kwargs: other keyword arguments used in BeseSession.send()
+        """
         if message:
             self._run_future(self.send(message, **kwargs))
-        raise _FinishException
+        self._raise(_FinishException())
 
     def switch(self, new_message: Message_T) -> NoReturn:
         """
@@ -662,12 +723,21 @@ class CommandSession(BaseSession):
 
         if not isinstance(new_message, Message):
             new_message = Message(new_message)
-        raise SwitchException(new_message)
+        self._raise(SwitchException(new_message))
+
+    def _raise(self, e: Exception) -> NoReturn:
+        """Raise an exception from the main execution path"""
+        if self.waiting:
+            self._future.set_exception(e)
+            raise _YieldException
+        raise e
 
 
 async def handle_command(bot: NoneBot, event: CQEvent,
                          manager: CommandManager) -> Optional[bool]:
     """
+    INTERNAL API
+
     Handle a message as a command.
 
     This function is typically called by "handle_message".
@@ -719,7 +789,7 @@ async def handle_command(bot: NoneBot, event: CQEvent,
             check_perm = False
         else:
             # the session is expired, remove it
-            logger.debug(f'Session of command {session.cmd.name} is expired')
+            logger.debug(f'Session of command {session.cmd.name} has expired')
             if ctx_id in _sessions:
                 del _sessions[ctx_id]
             session = None
@@ -754,7 +824,7 @@ async def call_command(bot: NoneBot,
     Call a command internally.
 
     This function is typically called by some other commands
-    or "handle_natural_language" when handling NLPResult object.
+    or "handle_natural_language" when handling IntentCommand object.
 
     Note: If disable_interaction is not True, after calling this function,
     any previous command session will be overridden, even if the command
@@ -792,17 +862,16 @@ async def _real_run_command(session: CommandSession,
         logger.debug(f'Running command {session.cmd.name}')
         session.running = True
         future = asyncio.ensure_future(session.cmd.run(session, **kwargs))
-        timeout = None
-        if session.bot.config.SESSION_RUN_TIMEOUT:
-            timeout = session.bot.config.SESSION_RUN_TIMEOUT.total_seconds()
+        timeout_opt = session.run_timeout
+        timeout = timeout_opt.total_seconds() if timeout_opt else None
 
         try:
             await asyncio.wait_for(future, timeout)
             handled = future.result()
         except asyncio.TimeoutError:
             handled = True
-        except (_PauseException, _FinishException, SwitchException) as e:
-            raise e
+        except CommandInterrupt:
+            raise
         except Exception as e:
             logger.error(f'An exception occurred while '
                          f'running command {session.cmd.name}:')
@@ -818,6 +887,9 @@ async def _real_run_command(session: CommandSession,
                      f'command {session.cmd.name}')
         # return True because this step of the session is successful
         return True
+    except _YieldException:
+        return True
+        # return True because this step of the session is successful
     except (_FinishException, SwitchException) as e:
         session.running = False
         logger.debug(f'Session of command {session.cmd.name} finished')
@@ -850,7 +922,18 @@ def kill_current_session(event: CQEvent) -> None:
     """
     ctx_id = context_id(event)
     if ctx_id in _sessions:
+        if _sessions[ctx_id].waiting:
+            _sessions[ctx_id]._future.set_exception(_FinishException())
         del _sessions[ctx_id]
 
 
-from nonebot.command.group import CommandGroup  # noqa: F401
+from nonebot.command.group import CommandGroup
+
+
+__all__ = [
+    'CommandManager',
+    'CommandSession',
+    'call_command',
+    'kill_current_session',
+    'CommandGroup',
+]
