@@ -1,7 +1,7 @@
 from typing import Optional, List
 
 
-def _simple_chinese_to_bool(text: str) -> Optional[bool]:
+def simple_chinese_to_bool(text: str) -> Optional[bool]:
     """
     Convert a chinese text to boolean.
 
@@ -28,15 +28,17 @@ def _simple_chinese_to_bool(text: str) -> Optional[bool]:
     return None
 
 
-def _split_nonempty_lines(text: str) -> List[str]:
+def split_nonempty_lines(text: str) -> List[str]:
     return list(filter(lambda x: x, text.splitlines()))
 
 
-def _split_nonempty_stripped_lines(text: str) -> List[str]:
+def split_nonempty_stripped_lines(text: str) -> List[str]:
     return list(filter(lambda x: x, map(lambda x: x.strip(),
                                         text.splitlines())))
 
 
-simple_chinese_to_bool = _simple_chinese_to_bool
-split_nonempty_lines = _split_nonempty_lines
-split_nonempty_stripped_lines = _split_nonempty_stripped_lines
+__all__ = [
+    'simple_chinese_to_bool',
+    'split_nonempty_lines',
+    'split_nonempty_stripped_lines',
+]
