@@ -1,7 +1,7 @@
 import asyncio
 import hashlib
 import random
-from typing import Sequence, Any
+from typing import Callable, Iterable, List, Sequence, Any, Tuple
 
 from aiocqhttp.message import escape
 from aiocqhttp import Event as CQEvent
@@ -98,14 +98,40 @@ def render_expression(expr: Expression_T,
     :param kwargs: keyword arguments used in str.format()
     :return: the rendered message
     """
+    result: str
     if callable(expr):
-        expr = expr(*args, **kwargs)
+        result = expr(*args, **kwargs)
     elif isinstance(expr, Sequence) and not isinstance(expr, str):
-        expr = random.choice(expr)
+        result = random.choice(expr)
+    else:
+        result = expr
     if escape_args:
-        return expr.format(
+        return result.format(
             *[escape(s) if isinstance(s, str) else s for s in args], **{
                 k: escape(v) if isinstance(v, str) else v
                 for k, v in kwargs.items()
             })
-    return expr.format(*args, **kwargs)
+    return result.format(*args, **kwargs)
+
+
+# TODO: ParamSpec
+def separate_async_funcs(funcs: Iterable[Callable]
+                         ) -> Tuple[List[Callable], List[Callable]]:
+    """INTERNAL API"""
+    syncs = []
+    asyncs = []
+    for f in funcs:
+        if asyncio.iscoroutinefunction(f) or (
+            asyncio.iscoroutinefunction(f.__call__)):
+            asyncs.append(f)
+        else:
+            syncs.append(f)
+    return syncs, asyncs
+
+
+__all__ = [
+    'context_id',
+    'send',
+    'send_to_superusers',
+    'render_expression',
+]
