@@ -1,22 +1,63 @@
 import re
 import asyncio
-from typing import Callable, Iterable
+import warnings
+from typing import Optional, Set, Iterable
 
 from aiocqhttp import Event as CQEvent
-from aiocqhttp.message import escape, unescape  # noqa: F401
 from aiocqhttp.message import Message, MessageSegment
+from aiocqhttp.message import escape, unescape
 
 from . import NoneBot
 from .log import logger
 from .natural_language import handle_natural_language
 from .command import handle_command, SwitchException
-from .plugin import PluginManager
-
-_message_preprocessors = set()
+from .plugin import Plugin, PluginManager
+from .typing import MessagePreprocessor_T
 
 
-def message_preprocessor(func: Callable) -> Callable:
-    _message_preprocessors.add(func)
+class MessagePreprocessor:
+    """INTERNAL_API"""
+    __slots__ = ('func',)
+
+    def __init__(self, func: MessagePreprocessor_T):
+        self.func = func
+
+
+class MessagePreprocessorManager:
+    """INTERNAL API"""
+    preprocessors: Set[MessagePreprocessor] = set()
+
+    @classmethod
+    def add_message_preprocessor(cls, preprocessor: MessagePreprocessor) -> None:
+        if preprocessor in cls.preprocessors:
+            warnings.warn(f"Message preprocessor {preprocessor} already exists")
+            return
+        cls.preprocessors.add(preprocessor)
+
+    @classmethod
+    def remove_message_preprocessor(cls, preprocessor: MessagePreprocessor) -> None:
+        cls.preprocessors.discard(preprocessor)
+
+    @classmethod
+    def switch_message_preprocessor_global(cls,
+                                          preprocessor: MessagePreprocessor,
+                                          state: Optional[bool] = None) -> None:
+        if preprocessor in cls.preprocessors and not state:
+            cls.preprocessors.discard(preprocessor)
+        elif preprocessor not in cls.preprocessors and state is not False:
+            cls.preprocessors.add(preprocessor)
+
+
+# this is more consistent if it is in the plugin module, but still kept here for historical
+# reasons
+def message_preprocessor(func: MessagePreprocessor_T) -> MessagePreprocessor_T:
+    mp = MessagePreprocessor(func)
+    if Plugin.GlobalTemp.now_within_plugin:
+        Plugin.GlobalTemp.msg_preprocessors.add(mp)
+    else:
+        warnings.warn('defining message_preprocessor outside a plugin is deprecated '
+                      'and will not be supported in the future')
+        MessagePreprocessorManager.add_message_preprocessor(mp)
     return func
 
 
@@ -34,6 +75,7 @@ class CanceledException(Exception):
 
 
 async def handle_message(bot: NoneBot, event: CQEvent) -> None:
+    """INTERNAL API"""
     _log_message(event)
 
     assert isinstance(event.message, Message)
@@ -47,8 +89,8 @@ async def handle_message(bot: NoneBot, event: CQEvent) -> None:
 
     coros = []
     plugin_manager = PluginManager()
-    for preprocessor in _message_preprocessors:
-        coros.append(preprocessor(bot, event, plugin_manager))
+    for preprocessor in MessagePreprocessorManager.preprocessors:
+        coros.append(preprocessor.func(bot, event, plugin_manager))
     if coros:
         try:
             await asyncio.gather(*coros)
@@ -80,33 +122,36 @@ async def handle_message(bot: NoneBot, event: CQEvent) -> None:
 def _check_at_me(bot: NoneBot, event: CQEvent) -> None:
     if event.detail_type == 'private':
         event['to_me'] = True
-    else:
-        # group or discuss
-        event['to_me'] = False
-        at_me_seg = MessageSegment.at(event.self_id)
+        return
 
-        # check the first segment
-        first_msg_seg = event.message[0]
-        if first_msg_seg == at_me_seg:
-            event['to_me'] = True
-            del event.message[0]
+    def is_at_me(seg):
+        return seg.type == 'at' and str(seg.data['qq']) == str(event.self_id)
 
-        if not event['to_me']:
-            # check the last segment
-            i = -1
+    # group or discuss
+    event['to_me'] = False
+
+    # check the first segment
+    first_msg_seg = event.message[0]
+    if is_at_me(first_msg_seg):
+        event['to_me'] = True
+        del event.message[0]
+
+    if not event['to_me']:
+        # check the last segment
+        i = -1
+        last_msg_seg = event.message[i]
+        if last_msg_seg.type == 'text' and \
+                not last_msg_seg.data['text'].strip() and \
+                len(event.message) >= 2:
+            i -= 1
             last_msg_seg = event.message[i]
-            if last_msg_seg.type == 'text' and \
-                    not last_msg_seg.data['text'].strip() and \
-                    len(event.message) >= 2:
-                i -= 1
-                last_msg_seg = event.message[i]
 
-            if last_msg_seg == at_me_seg:
-                event['to_me'] = True
-                del event.message[i:]
+        if is_at_me(last_msg_seg):
+            event['to_me'] = True
+            del event.message[i:]
 
-        if not event.message:
-            event.message.append(MessageSegment.text(''))
+    if not event.message:
+        event.message.append(MessageSegment.text(''))
 
 
 def _check_calling_me_nickname(bot: NoneBot, event: CQEvent) -> None:
@@ -142,3 +187,13 @@ def _log_message(event: CQEvent) -> None:
     logger.info(f'Self: {event.self_id}, '
                 f'Message {event.message_id} from {msg_from}: '
                 f'{repr(str(event.message))}')
+
+
+__all__ = [
+    'message_preprocessor',
+    'CanceledException',
+    'Message',
+    'MessageSegment',
+    'escape',
+    'unescape',
+]
