@@ -1,33 +1,84 @@
 import asyncio
 import warnings
-from typing import Set, Iterable, Optional, Callable, Union, NamedTuple
+from typing import Any, List, Set, Iterable, Optional, Tuple, Union, NamedTuple
 
 from aiocqhttp import Event as CQEvent
 from aiocqhttp.message import Message
 
 from .log import logger
-from . import NoneBot, permission as perm
+from . import NoneBot
 from .command import call_command
 from .session import BaseSession
-from .typing import CommandName_T, CommandArgs_T
+from .typing import CommandName_T, CommandArgs_T, NLPHandler_T, PermissionPolicy_T
+from .permission import check_permission
 
 
 class NLProcessor:
-    __slots__ = ('func', 'keywords', 'permission', 'only_to_me',
-                 'only_short_message', 'allow_empty_message')
+    """INTERNAL API"""
+    __slots__ = ('func', 'keywords', 'only_to_me', 'only_short_message',
+                 'allow_empty_message', 'permission')
 
-    def __init__(self, *, func: Callable, keywords: Optional[Iterable],
-                 permission: int, only_to_me: bool, only_short_message: bool,
-                 allow_empty_message: bool):
+    def __init__(self, *, func: NLPHandler_T, keywords: Optional[Iterable[str]],
+                 only_to_me: bool, only_short_message: bool,
+                 allow_empty_message: bool,
+                 permission: PermissionPolicy_T):
         self.func = func
         self.keywords = keywords
-        self.permission = permission
         self.only_to_me = only_to_me
         self.only_short_message = only_short_message
         self.allow_empty_message = allow_empty_message
+        self.permission = permission  # includes EllipsisType
+
+    async def test(self, session: 'NLPSession', 
+                   msg_text_length: Optional[int] = None) -> bool:
+        """
+        Test whether the session matches this (self) NL processor.
+
+        :param session: NLPSession object
+        :param msg_text_length: this argument is `len(session.msg_text)`,
+                                designated to be cached if this function
+                                is invoked in a loop
+        :return: the session context matches this processor
+        """
+        if msg_text_length is None:
+            msg_text_length = len(session.msg_text)
+
+        if not self.allow_empty_message and not session.msg:
+            # don't allow empty msg, but it is one, so no
+            return False
+
+        if self.only_short_message and \
+            msg_text_length > session.bot.config.SHORT_MESSAGE_MAX_LENGTH:
+            return False
+
+        if self.only_to_me and not session.event['to_me']:
+            return False
+
+        if self.keywords:
+            for kw in self.keywords:
+                if kw in session.msg_text:
+                    break
+            else:
+                # no keyword matches
+                return False
+
+        return await self._check_perm(session)
+
+    async def _check_perm(self, session: 'NLPSession') -> bool:
+        """
+        Check if the session has sufficient permission to
+        call the command.
+
+        :param session: NLPSession object
+        :return: the event has the permission
+        """
+        return await check_permission(session.bot, session.event,
+            self.permission if self.permission is not ...
+                else session.bot.config.DEFAULT_NLP_PERMISSION)
 
 
 class NLPManager:
+    """INTERNAL API"""
     _nl_processors: Set[NLProcessor] = set()
 
     def __init__(self):
@@ -114,21 +165,6 @@ class NLPSession(BaseSession):
         ]
 
 
-class NLPResult(NamedTuple):
-    """
-    Deprecated.
-    Use class IntentCommand instead.
-    """
-    confidence: float
-    cmd_name: Union[str, CommandName_T]
-    cmd_args: Optional[CommandArgs_T] = None
-
-    def to_intent_command(self):
-        return IntentCommand(confidence=self.confidence,
-                             name=self.cmd_name,
-                             args=self.cmd_args)
-
-
 class IntentCommand(NamedTuple):
     """
     To represent a command that we think the user may be intended to call.
@@ -142,6 +178,8 @@ class IntentCommand(NamedTuple):
 async def handle_natural_language(bot: NoneBot, event: CQEvent,
                                   manager: NLPManager) -> bool:
     """
+    INTERNAL API
+
     Handle a message as natural language.
 
     This function is typically called by "handle_message".
@@ -157,60 +195,52 @@ async def handle_natural_language(bot: NoneBot, event: CQEvent,
     # at the same time some plugins may want to handle it
     msg_text_length = len(session.msg_text)
 
-    futures = []
-    for p in manager.nl_processors:
-        if not p.allow_empty_message and not session.msg:
-            # don't allow empty msg, but it is one, so skip to next
+    # returns 1. processor result; 2. whether this processor is considered handled
+    async def try_run_nlp(p: NLProcessor) -> Tuple[Any, bool]:
+        try:
+            should_run = await p.test(session, msg_text_length=msg_text_length)
+            if should_run:
+                return await p.func(session), True
+            return None, False
+        except Exception as e:
+            logger.error('An exception occurred while running '
+                         'some natural language processor:')
+            logger.exception(e)
+            return None, True
+
+    intent_commands: List[IntentCommand] = []
+    procs_empty = True
+
+    for res in asyncio.as_completed([try_run_nlp(p) for p in manager.nl_processors]):
+        result, should_run = await res
+        if not should_run:
             continue
+        procs_empty = False
+        if isinstance(result, IntentCommand):
+            intent_commands.append(result)
 
-        if p.only_short_message and \
-                msg_text_length > bot.config.SHORT_MESSAGE_MAX_LENGTH:
-            continue
+    if procs_empty:
+        return False
 
-        if p.only_to_me and not event['to_me']:
-            continue
+    intent_commands.sort(key=lambda ic: ic.confidence, reverse=True)
+    logger.debug(f'Intent commands: {intent_commands}')
 
-        should_run = await perm.check_permission(bot, event, p.permission)
-        if should_run and p.keywords:
-            for kw in p.keywords:
-                if kw in session.msg_text:
-                    break
-            else:
-                # no keyword matches
-                should_run = False
-
-        if should_run:
-            futures.append(asyncio.ensure_future(p.func(session)))
-
-    if futures:
-        # wait for intent commands, and sort them by confidence
-        intent_commands = []
-        for fut in futures:
-            try:
-                res = await fut
-                if isinstance(res, NLPResult):
-                    intent_commands.append(res.to_intent_command())
-                elif isinstance(res, IntentCommand):
-                    intent_commands.append(res)
-            except Exception as e:
-                logger.error('An exception occurred while running '
-                             'some natural language processor:')
-                logger.exception(e)
-
-        intent_commands.sort(key=lambda ic: ic.confidence, reverse=True)
-        logger.debug(f'Intent commands: {intent_commands}')
-
-        if intent_commands and intent_commands[0].confidence >= 60.0:
-            # choose the intent command with highest confidence
-            chosen_cmd = intent_commands[0]
-            logger.debug(
-                f'Intent command with highest confidence: {chosen_cmd}')
-            return await call_command(bot,
-                                      event,
-                                      chosen_cmd.name,
-                                      args=chosen_cmd.args,
-                                      current_arg=chosen_cmd.current_arg,
-                                      check_perm=False)  # type: ignore
-        else:
-            logger.debug('No intent command has enough confidence')
+    if intent_commands and intent_commands[0].confidence >= 60.0:
+        # choose the intent command with highest confidence
+        chosen_cmd = intent_commands[0]
+        logger.debug(
+            f'Intent command with highest confidence: {chosen_cmd}')
+        return await call_command(bot,
+                                  event,
+                                  chosen_cmd.name,
+                                  args=chosen_cmd.args,
+                                  current_arg=chosen_cmd.current_arg,
+                                  check_perm=False) or False
+    logger.debug('No intent command has enough confidence')
     return False
+
+
+__all__ = [
+    'NLPSession',
+    'IntentCommand',
+]
